

def url_link_button(url, label="Authenticate", **kwargs):
    import panel as pn

    button = pn.widgets.Button(name=label, **kwargs)
    button.js_on_click(code=f"window.open('{url}')")
    return button


def id_token_from_server_state():
    import panel as pn
    from tornado.web import decode_signed_value

    id_token = pn.state.cookies.get('id_token')
    if id_token is None or pn.config.cookie_secret is None:
        return None
    id_token = decode_signed_value(pn.config.cookie_secret, 'id_token', id_token)
    if pn.state.encryption is None:
        id_token = id_token
    else:
        id_token = pn.state.encryption.decrypt(id_token)
    return id_token.decode()


import time
import param
import httpx

from .settings import config
from .token import XeToken
from .oauth import XeAuthStep


class XeTokenRequest(XeAuthStep):

    oauth_domain = param.String(config.OAUTH_DOMAIN)
    oauth_token_path = param.String(config.OAUTH_TOKEN_PATH)
    user_code = param.String()
    device_code = param.String()
    client_id = param.String()
    headers = param.Dict()
    
    verification_uri = param.String()
    verification_uri_complete = param.String()
    
    expires = param.Number()
    interval = param.Number(5)

    open_browser = param.Boolean(True)
    
    def prompt(self, p):
        print(f'Please visit the following URL to complete ' 
              f'the login: {self.verification_uri_complete}', file=p.console)
        if p.open_browser:
            import webbrowser
            webbrowser.open(self.verification_uri_complete)
        return p

    def perform(self, p):
        while True:
            if time.time()>p.expires:
                raise TimeoutError("Device code hase expired but not yet authorized.")
            try:
                s = self.fetch_token(p.oauth_domain, p.oauth_token_path, 
                                     p.device_code, p.client_id, headers=p.headers)
                
                return s
            except Exception as e:
                time.sleep(p.interval)
           
    def fetch_token(self, oauth_domain, oauth_token_path, device_code, client_id, headers={}):
        with httpx.Client(base_url=oauth_domain, headers=headers) as client:
            r = client.post(
                oauth_token_path,
                
            data={
                "grant_type": "urn:ietf:params:oauth:grant-type:device_code",
                "device_code": device_code,
                "client_id": client_id,
            },
            headers={"content-type": "application/x-www-form-urlencoded"},
            )
            r.raise_for_status()
            params = r.json()
            params["expires"] = time.time() + params.pop("expires_in", 1e6)
            params["client_id"] = self.client_id
            params['oauth_domain'] = oauth_domain
            params['oauth_token_path'] = oauth_token_path
            
        return XeToken(**params)


class XeAuthCodeRequest(XeAuthStep):
    oauth_domain = param.String(config.OAUTH_DOMAIN)
    oauth_code_path = param.String(config.OAUTH_CODE_PATH)
    client_id = param.String(config.DEFAULT_CLIENT_ID) 
    scopes = param.List(config.DEFAULT_SCOPE.split(' '))
    audience = param.String(config.DEFAULT_AUDIENCE)
    extra_fields = param.Dict({})
    headers = param.Dict({})
    
    @property
    def scope_str(self):
        return ' '.join(self.scopes)
    
    def perform(self, p):
        data = {
                    "client_id": p.client_id,
                    "scope": ' '.join(p.scopes),
                    "audience": p.audience,
                    }
        data.update(p.extra_fields)
        
        with httpx.Client(base_url=p.oauth_domain, headers=p.headers) as client:
    
            r = client.post(
                p.oauth_code_path,
                data=data,
                headers={"content-type": "application/x-www-form-urlencoded"})
            
            r.raise_for_status()
            
        params = r.json()
        
        params['expires'] = time.time() + params.pop("expires_in", 1)
        params['oauth_domain'] = p.oauth_domain
        params['client_id'] = p.client_id

        return XeTokenRequest.instance(**params)




import io
import sys
import param


class XeAuthStep(param.ParameterizedFunction):
    auto_advance = param.Boolean(True)
    prompt_response = param.Parameter(instantiate=False)
    console = param.ClassSelector(io.IOBase, default=sys.stdout, 
                                  instantiate=False, pickle_default_value=False)

    def perform(self, p):
        pass

    def prompt(self, p):
        return p
    
    def __call__(self, **params):
        p = param.ParamOverrides(self, params)

        p = self.prompt(p)
        
        next = self.perform(p)

        if isinstance(next, XeAuthStep) and p.auto_advance:
            params = {k:v for k,v in params.items() if k in next.param.params()}
            next = next(**params)
        return next




import param
import httpx
import time
import getpass

from .oauth import XeAuthStep
from .token import XeToken
from .settings import config

class UserCredentialsAuth(XeAuthStep):
    username = param.String(default=None)
    password = param.String(default=None)

    auth_url = param.String(config.OAUTH_DOMAIN.rstrip('/')+'/token')
    audience = param.String(config.DEFAULT_AUDIENCE)
    scopes = param.List(config.DEFAULT_SCOPE.split(' '))
    client_id = param.String(config.DEFAULT_CLIENT_ID)
    headers = param.Dict({'content-type': 'application/x-www-form-urlencoded'})

    def prompt(self, p):
        if p.username is None:
            p.username = getpass.getuser()
        if p.password is None:
            p.password = getpass.getpass()
        return p

    def perform(self, p):
        data = dict(
            grant_type='password',
            username=p.username,
            password=p.password,
            audience=p.audience,
            scope=' '.join(p.scopes),
            client_id=p.client_id,
        )
        r = httpx.post(p.auth_url, data=data, headers=p.headers)
        r.raise_for_status()
        kwargs = r.json()
        kwargs['expires'] = time.time() + kwargs.pop('expires_in')
        return XeToken(client_id=p.client_id, **kwargs)



import param
import httpx
import authlib
from authlib import jose
from authlib.jose import KeySet
import time
from contextlib import contextmanager
import logging

from .settings import config


logger = logging.getLogger(__name__)

class XeKeySet(param.Parameterized):
    oauth_domain = param.String(config.OAUTH_DOMAIN)
    cert_path = param.String(config.OAUTH_CERT_PATH)

    _keyset = param.ClassSelector(KeySet, default=KeySet({}))

    _keys_timestamp = param.Number(0)
    _keys_ttl = param.Number(300)

    def fetch_keys(self, headers={}):
        with httpx.Client(base_url=self.oauth_domain, headers=headers) as client:
            r = client.get(self.cert_path)
            r.raise_for_status()
        keys = r.json()
        self._keyset = jose.JsonWebKey.import_key_set(keys)

    def extract_claims(self, token):
        header_str = authlib.common.encoding.urlsafe_b64decode(token.split(".")[0].encode()).decode('utf-8')
        header = authlib.common.encoding.json_loads(header_str)
        key = self.find_by_kid(header["kid"])
        return jose.jwt.decode(token, key)

    def extract_verified_claims(self, token, options={}):
        try:
            claims = self.extract_claims(token)
            claims.options = options
            claims.validate()
            return claims
            
        except Exception as e:
            logger.error(f"Exception raised while validating claims: {e}")
            return jose.JWTClaims("", "")

    def validate_claims(self, token, **required_claims):
        options = {k: {"value": v, "essential": True} for k,v in required_claims.items()}
        claims = self.extract_claims(token)
        claims.options = options
        claims.validate()

    def find_by_kid(self, kid):
        if kid not in self._keyset.keys:
            self.fetch_keys()
        return self._keyset.find_by_kid(kid)

    def __getitem__(self, kid):
        return self.find_by_kid(kid)

certs = XeKeySet()



import os

from .settings import config

from .user_credentials import UserCredentialsAuth
from .device_auth_flow import XeAuthCodeRequest
from .certificates import certs


user_login = UserCredentialsAuth.instance(auto_advance=True)


device_login = XeAuthCodeRequest.instance(auto_advance=True)


def login(username=None, password=None, **kwargs):

    if username is None:
        return device_login(**kwargs)
    
    return user_login(username=username,
                      password=password,
                      **kwargs)
    

def cli_login(**kwargs):
    token = login(**kwargs)
    print(f"logged in as: {token.profile.get('name', 'unknown')}")
    print(f"Access token: {token.access_token}")
    print(f"ID token: {token.id_token}")


def validate_claims(token, **claims):
    return certs.validate_claims(token, **claims)

def clear_cache():
    os.remove(config.CACHE_FILE)

def cmt_login(scopes=[], **kwargs):
    if isinstance(scopes, str):
        scopes = scopes.split(' ')
    if not isinstance(scopes, list):
        raise TypeError('scopes must be a list or string')
    scopes = set(scopes)
    scopes.add('read:all')
    scopes =  list(scopes)
    audience = kwargs.pop('audience', 'https://api.cmt.xenonnt.org')
    # base_url = kwargs.pop('base_url', DEFAULT_BASE_URL)
    return login(audience=audience, scopes=scopes, **kwargs)




import os
import param
import httpx
import time
import webbrowser
import logging

from datetime import datetime
from contextlib import contextmanager, asynccontextmanager

from .settings import config
from .utils import url_link_button, id_token_from_server_state
from .certificates import certs
from .token import XeToken
from .device_auth_flow import DeviceAuthFlow

logger = logging.getLogger(__name__)


class XeAuthSession(param.Parameterized):
    keyset = certs
    
    oauth_domain = param.String(config.OAUTH_DOMAIN)
    oauth_code_path = param.String(config.OAUTH_CODE_PATH)
    oauth_token_path = param.String(config.OAUTH_TOKEN_PATH)
    
    auto_persist_session = param.Boolean(False)
    token_file = param.String(config.TOKEN_FILE)
    client_id = param.String(config.DEFAULT_CLIENT_ID) 
    scopes = param.List([])
    audience = param.String(config.DEFAULT_AUDIENCE)

    notify_email = param.String(allow_None=True)

    flow = param.ClassSelector(default=DeviceAuthFlow, class_=DeviceAuthFlow, is_instance=False)
    
    token = param.ClassSelector(XeToken, default=None)
    state = param.Selector(["Disconnected", "Logged in", "Awaiting token",
                            "Checking token ready", "Token expired"],
                             default="Disconnected")

    def __init__(self, **params):
        super().__init__(**params)
        if os.path.isfile(self.token_file):
            self.token = XeToken.from_file(self.token_file)
            if self.logged_in:
                self.state = "Logged in"

    def login_from_server(self):
        import panel as pn

        access_token = pn.state.access_token
        id_token = id_token_from_server_state()
        self.token = XeToken(access_token=access_token,
                             id_token=id_token,
                            )
        return self.token

    @property
    def scope(self):
        scopes = set(config.DEFAULT_SCOPE.split(" ") + self.scopes)
        return " ".join(scopes)

    def login(self, open_browser=False, print_url=False, extra_headers={}, notify_email=None):
        extra_fields = {}
        if notify_email:
            extra_fields["notify_email"] = notify_email
        if self.token:
            self.state = "Logged in"
        else:
            self.token = self.flow(self.oauth_domain, self.oauth_code_path, self.oauth_token_path, 
                                            self.client_id, self.scope, self.audience, headers=extra_headers,
                                            extra_fields=extra_fields,
                                        open_browser=open_browser, print_url=print_url)
            if self.token:
                self.state = "Logged in"
        return self.token

    def persist_token(self):
        if self.token_file:
            try:
                self.token.to_file(self.token_file)
            except Exception as e:
                logger.error("Exception raised while persisted token: "+str(e))

    def logout(self):
        self.token = None
        self.state = "Disconnected"
        return True

    def request_token(self, extra_headers={}):
        
        self.flow.request_token(self.oauth_domain, self.oauth_code_path, 
                                self.client_id, self.scope, self.audience, headers=extra_headers)
        self.state = "Awaiting token"

    def token_ready(self, extra_headers={}):
        if self.token is None:
            try:
                self.token = self.flow.fetch_token(self.oauth_domain,
                                         self.oauth_token_path, headers=extra_headers)
                if self.auto_persist_session:
                    self.persist_token()
                    
                self.state = "Logged in"
                return True
            except:
                pass
        return False
    
    def refresh_tokens(self, extra_headers={}):
        self.token.refresh_tokens(self.oauth_domain, self.oauth_token_path,
                                             self.client_id, headers=extra_headers)
        if self.auto_persist_session:
            self.persist_token()

    @property
    def logged_in(self):
        if self.token is None: 
            return False
        if self.token.expired:
            return False
        return True

    @property
    def id_token(self):
        return self.token.id_token

    @property
    def access_token(self):
        return self.token.access_token

    @property
    def profile(self):
        claims = self.keyset.extract_verified_claims(self.id_token)
        return {k:v for k,v in claims.items() if k not in claims.REGISTERED_CLAIMS}
    
    @property
    def claims(self):
        claims = self.keyset.extract_verified_claims(self.access_token)
        return {k:v for k,v in claims.items() if k in claims.REGISTERED_CLAIMS}

    @property
    def extra_claims(self):
        claims = self.keyset.extract_verified_claims(self.access_token)
        return {k:v for k,v in claims.items() if k not in claims.REGISTERED_CLAIMS}

    @property
    def permissions(self):
        claims = self.keyset.extract_verified_claims(self.access_token)
        return claims.get("permissions", [])

    @contextmanager
    def Client(self, *args, **kwargs):
        kwargs["headers"] = kwargs.get("headers", {})
        kwargs["headers"]["Authorization"] = f"Bearer {self.access_token}"
        
        client = httpx.Client(*args, **kwargs)
        try:
            yield client
        finally:
            client.close()

    @asynccontextmanager
    async def AsyncClient(self, *args, **kwargs ):
        kwargs["headers"] = kwargs.get("headers", {})
        kwargs["headers"]["Authorization"] = f"Bearer {self.access_token}"
        client = httpx.AsyncClient(*args, **kwargs)
        try:
            yield client
        finally:
            await client.aclose()

    def authorize(self):
        webbrowser.open(self.flow.verification_uri_complete)
        
class NotebookSession(XeAuthSession):
    message = param.String("")
    _gui = None
    _cb = None

    @property
    def gui(self):
        import panel as pn
        if self._gui is None:
            self._gui = pn.panel(self._make_gui)
        return self._gui

    def await_token_cb(self):
        if self.token_ready() and self.token:
            self._cb.stop()
        
    def login_requested(self, event):
        try:
            import panel as pn
            self.request_token()
            logger.info("Sent request...")
            self._cb = pn.state.add_periodic_callback(self.await_token_cb,
                                                    1000*self.flow.interval,
                                                    timeout=1000*max(1, int(self.flow.expires-time.time())))
        except Exception as e:
            logging.error(e)
            print(e)

    def logged_in_gui(self):
        import panel as pn
        profile = self.profile
        details = pn.Row(
            pn.pane.PNG(profile.get("picture", config.DEFAULT_AVATAR), width=60, height=60),
            pn.widgets.TextInput(name='Name', value=profile.get("name", "Unknown"), disabled=True, height=35),
            pn.widgets.TextInput(name='Email', value=profile.get("email", "Unknown"), disabled=True, height=35),
            
            height=70
        )
        token = pn.widgets.input.TextAreaInput(name='Access token', value=self.access_token, width=700, height=100)
        
        token_props = pn.Row(
            pn.widgets.TextInput(name='Scope', value=self.token.scope, disabled=True, height=35),
            pn.widgets.DatetimeInput(disabled=True, name="Expiration date",
                         value=datetime.utcfromtimestamp(self.token.expires)),
            
            width=700,
        )
        logout = pn.widgets.Button(name="Logout", button_type='warning')
        logout.on_click(lambda event: self.logout())
        return pn.Column(
                    details,
                    token,
                    token_props,
                    logout,
                    height=300,
                    width=700)

    def awaiting_token_gui(self):
        import panel as pn

        cancel = pn.widgets.Button(name="Cancel", width=50)
        cancel.on_click(lambda event: self.logout())
        authenticate = url_link_button(self.flow.verification_uri_complete, button_type='primary', width=100)
        
        waiting_ind = pn.indicators.LoadingSpinner(value=False, width=30, height=30, align="center")
        def activate(event):
            waiting_ind.value = True
        authenticate.on_click(activate)
        return pn.Row(waiting_ind, authenticate, cancel)
    
    def token_expired_gui(self):
        import panel as pn

        refresh = pn.widgets.Button(name="Renew", align="end")
        refresh.on_click(lambda e: self.refresh_tokens())
        return refresh

    @param.depends("state")
    def _make_gui(self):
        import panel as pn

        status = pn.indicators.BooleanStatus(width=20, height=20, value=True, color="danger", align="center")
        header = pn.Row(status, f"Status: {self.state}.")
        panel = pn.Column(header)
        if self.state == "Logged in":
            status.color = "success"
            panel.append(self.logged_in_gui())

        elif self.state == "Awaiting token":
            status.color = "primary"
            panel.append(self.awaiting_token_gui())

        elif self.state == "Token expired":
            status.color = "warning"
            return pn.panel(self.token_expired_gui())

        else:
            login = pn.widgets.Button(name="Login", button_type='success', width=30)
            login.on_click(self.login_requested)
            panel.append(pn.Param(self.param, parameters=["scopes", "auto_persist_session"]))
            panel.append(login)

        return panel

    def _repr_mimebundle_(self, include=None, exclude=None):
        return self.gui._repr_mimebundle_(include=include, exclude=exclude)



import param
import time
import httpx
import json
from contextlib import contextmanager, asynccontextmanager

from .settings import config
from .utils import id_token_from_server_state
from .certificates import certs

class XeToken(param.Parameterized):
    client_id = param.String(config.DEFAULT_CLIENT_ID)
    oauth_domain = param.String(config.OAUTH_DOMAIN)
    oauth_token_path = param.String(config.OAUTH_TOKEN_PATH)

    access_token = param.String(constant=True)
    id_token = param.String(constant=True)
    refresh_token = param.String(constant=True)
    expires = param.Number(constant=True)
    scope = param.String(constant=True)
    token_type = param.String("Bearer", constant=True)

    @property
    def expired(self):
        return time.time()>self.expires

    @property
    def profile(self):
        claims = certs.extract_verified_claims(self.id_token)
        return {k:v for k,v in claims.items() if k not in claims.REGISTERED_CLAIMS}
    
    @property
    def username(self):
        return self.profile.get('name', 'unknown')
        
    @property
    def claims(self):
        claims = certs.extract_verified_claims(self.access_token)
        return {k:v for k,v in claims.items() if k in claims.REGISTERED_CLAIMS}

    @property
    def extra_claims(self):
        claims = certs.extract_verified_claims(self.access_token)
        return {k:v for k,v in claims.items() if k not in claims.REGISTERED_CLAIMS}

    @property
    def permissions(self):
        claims = certs.extract_verified_claims(self.access_token)
        return claims.get("permissions", [])

    @classmethod
    def from_file(cls, path):
        with open(path, "r") as f:
            data = json.load(f)
        return cls(**data)

    @classmethod
    def from_panel_server(cls):
        import panel as pn
        access_token = pn.state.access_token
        id_token = id_token_from_server_state()
        token = cls(access_token=access_token,
                             id_token=id_token,
                            )
        return token
        
    def to_file(self, path):
        with open(path, "w") as f:
            json.dump(self.to_dict(), f)

    def to_dict(self):
        return {k:v for k,v in self.param.get_param_values() if not k.startswith("_")}

    def refresh(self, headers={}):
        with httpx.Client(base_url=self.oauth_domain, headers=headers) as client:
            r = client.post(
                self.oauth_token_path,
            headers={"content-type":"application/x-www-form-urlencoded"},
            data={
                "grant_type": "refresh_token",
                "refresh_token": self.refresh_token,
                "client_id": self.client_id,
            }
            )
            r.raise_for_status()
            params = r.json()
            params["expires"] = time.time() + params.pop("expires_in", 1e6)
            self.param.set_param(**params)
    
    @contextmanager
    def Client(self, *args, **kwargs):
        kwargs["headers"] = kwargs.get("headers", {})
        kwargs["headers"]["Authorization"] = f"Bearer {self.access_token}"
        
        client = httpx.Client(*args, **kwargs)
        try:
            yield client
        finally:
            client.close()

    @asynccontextmanager
    async def AsyncClient(self, *args, **kwargs ):
        kwargs["headers"] = kwargs.get("headers", {})
        kwargs["headers"]["Authorization"] = f"Bearer {self.access_token}"

        client = httpx.AsyncClient(*args, **kwargs)
        try:
            yield client
        finally:
            await client.aclose()

    def __repr__(self):
        return ("XeToken("
               f"user={self.profile.get('name', 'unknown')}, "
               f"access_token={self.access_token})"
                )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tests', 'xeauth', 'xeauth.integrations']

package_data = \
{'': ['*']}

install_requires = \
['Authlib>=1.0.0,<2.0.0',
 'appdirs>=1.4.4,<2.0.0',
 'click',
 'httpx>=0.19,<0.23',
 'param>=1.12.0,<2.0.0']

entry_points = \
{'console_scripts': ['xeauth = xeauth.cli:main'],
 'eve_panel.auth': ['XenonAuth = xeauth.integrations.eve_panel:XenonEveAuth'],
 'panel.auth': ['xeauth = xeauth.integrations.panel_server:XenonPanelAuth']}

setup_kwargs = {
    'name': 'xeauth',
    'version': '0.1.21',
    'description': 'Top-level package for xeauth.',
    'long_description': '======\nxeauth\n======\n\n\n.. image:: https://img.shields.io/pypi/v/xeauth.svg\n        :target: https://pypi.python.org/pypi/xeauth\n\n.. image:: https://img.shields.io/travis/jmosbacher/xeauth.svg\n        :target: https://travis-ci.com/jmosbacher/xeauth\n\n.. image:: https://readthedocs.org/projects/xeauth/badge/?version=latest\n        :target: https://xeauth.readthedocs.io/en/latest/?badge=latest\n        :alt: Documentation Status\n\n\n\n\nAuthentication client for the Xenon edark matter experiment.\n\n\n* Free software: MIT\n* Documentation: https://xeauth.readthedocs.io.\n\n\nFeatures\n--------\n\n* TODO\n\nCredits\n-------\n\nThis package was created with Cookiecutter_ and the `briggySmalls/cookiecutter-pypackage`_ project template.\n\n.. _Cookiecutter: https://github.com/audreyr/cookiecutter\n.. _`briggySmalls/cookiecutter-pypackage`: https://github.com/briggySmalls/cookiecutter-pypackage\n',
    'author': 'Yossi Mosbacher',
    'author_email': 'joe.mosbacher@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/jmosbacher/xeauth',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<3.11',
}


setup(**setup_kwargs)



import param
import json
import os
import getpass
from appdirs import AppDirs


DIRS = AppDirs("xeauth", "XENON")
CACHE_DIR = DIRS.user_cache_dir
if not os.path.isdir(CACHE_DIR):
    os.mkdir(CACHE_DIR)
DEFAULT_TOKEN_FILE = os.path.join(CACHE_DIR, f"{getpass.getuser()}_xetoken.json")
 

class ConfigParameter(param.Parameter):

    __slots__ = ["env_prefix", "klass"]

    def __init__(self, klass, env_prefix="", **kwargs):
        super().__init__(**kwargs)
        self.env_prefix = env_prefix
        self.klass = klass
       
    def _set_names(self, attrib_name):
        env_name = attrib_name.upper()
        env_name = self.env_prefix.upper() + "_" + env_name
        if os.getenv(env_name, ""):
            env = os.getenv(env_name, "")
            try:
                env = json.loads(env)
            except Exception as e:
                pass
            self.default = self.klass(env)
        super()._set_names(attrib_name)
        

class Config(param.Parameterized):
    OAUTH_DOMAIN = ConfigParameter(str, env_prefix="xeauth", default="https://xenon-experiment.eu.auth0.com/oauth")
    OAUTH_CODE_PATH = ConfigParameter(str, env_prefix="xeauth", default="/device/code")
    OAUTH_TOKEN_PATH = ConfigParameter(str, env_prefix="xeauth", default="/token")
    OAUTH_CERT_PATH = ConfigParameter(str, env_prefix="xeauth", default="/.well-known/jwks.json")
    AUTH0_SUBDOMAIN = ConfigParameter(str, env_prefix="xeauth", default="xenon-experiment.eu")
    DEFAULT_CLIENT_ID = ConfigParameter(str, env_prefix="xeauth", default="EC3adX50KdNHQuEmib30GCRDTFDibMK7")
    DEFAULT_AUDIENCE = ConfigParameter(str, env_prefix="xeauth", default="https://users.xenonnt.org")
    DEFAULT_SCOPE = ConfigParameter(str, env_prefix="xeauth", default="openid profile email offline_access")
    DEBUG = ConfigParameter(bool, env_prefix="xeauth", default=False)
    MAX_LOG_SIZE = 20
    MAX_MESSAGES = 3
    META_FIELDS = ["_version", "_latest_version", "_etag", "_created"]
    GUI_WIDTH = 600
    DEFAULT_AVATAR = "http://www.sibberhuuske.nl/wp-content/uploads/2016/10/default-avatar.png"
    TOKEN_FILE = ConfigParameter(str, env_prefix="xeauth", default=DEFAULT_TOKEN_FILE)

config = Config()





import xeauth

try:
    from eve.auth import TokenAuth
    from eve.utils import config
except ImportError:
    class TokenAuth:
        def __init__(self, *args, **kwargs):
            raise RuntimeError('eve not installed.')

class XeTokenAuth(TokenAuth):

    @property
    def global_read_token(self):
        return config.get('API_GLOBAL_READ_TOKEN', None)

    @property
    def audience(self):
        return config.get('JWT_AUDIENCE', 'https//:api.pmts.xenonnt.org')

    def check_auth(self, token, allowed_roles, resource, method):
        if not token:
            return False
        if method in ['GET', 'HEAD'] and self.global_read_token==token:
            return True
        try:
            xeauth.certs.validate_claims(token, audience=self.audience)
        except:
            return False
        c = xeauth.certs.extract_verified_claims(token)
        roles = c.get('scope', '').split(' ')
        return any([r in allowed_roles for r in roles])


import time
import param
import httpx

from .settings import config
from .token import XeToken
from .oauth import XeAuthStep



class TokenRefresh(XeAuthStep):
    client_id = param.String(config.DEFAULT_CLIENT_ID)
    oauth_domain = param.String(config.OAUTH_DOMAIN)
    oauth_token_path = param.String(config.OAUTH_TOKEN_PATH)

    access_token = param.String(readonly=True)
    id_token = param.String(readonly=True)
    refresh_token = param.String(readonly=True)

    def perform(self, p):
        with httpx.Client(base_url=self.oauth_domain, headers=p.headers) as client:
            r = client.post(
                p.oauth_token_path,
            headers={"content-type":"application/x-www-form-urlencoded"},
            data={
                "grant_type": "refresh_token",
                "refresh_token": p.refresh_token,
                "client_id": p.client_id,
            }
            )
            r.raise_for_status()
            params = r.json()
            params["expires"] = time.time() + params.pop("expires_in", 1e6)
            params["client_id"] = p.client_id
            params['oauth_domain'] = p.oauth_domain
            params['oauth_token_path'] = p.oauth_token_path
            return XeToken(**params)




from warnings import warn
import param

from ..session import NotebookSession
try:
    from eve_panel.auth import EveAuthBase
except ImportError:
    class EveAuthBase:
        def __init__(self) -> None:
            raise RuntimeError('eve_panel not installed.')

class XenonEveAuth(NotebookSession, EveAuthBase):
    # session = param.ClassSelector(NotebookSession, default=NotebookSession())

    def get_headers(self):
        """Generate auth headers for HTTP requests.

        Returns:
            dict: Auth related headers to be included in all requests.
        """
        if self.logged_in:
            return {"Authorization": f"Bearer {self.access_token}"}
        else:
            return {}

    def login(self, notify_email=None):
        """perform any actions required to aquire credentials.

        Returns:
            bool: whether login was successful
        """
        self.login_requested(None)
        self.authorize()

    def set_credentials(self, **credentials):
        """Set the access credentials manually.
        """
        for k,v in credentials.items():
            if k in ['access_token', "id_token", "refresh_token", "expires"]:
                setattr(self.token, k, v)
            else:
                setattr(self, k, v)
                
    def credentials_view(self):
        return self.gui
